from Reader import Reader
from matplotlib.figure import Figure
import numpy as np


class Analysis:
    data = Reader.read_csv("Cleaned_Laptop_data.csv")

    @staticmethod
    def get_brand_list():
        brand_list = []
        pre_list = Analysis.data["brand"].unique()
        for each_brand in pre_list:
            brand_list.append(each_brand)
        return brand_list

    @staticmethod
    def get_model_list_by_brand(brand):
        filtered_df = Analysis.data[Analysis.data["brand"] == brand]
        model_list = []
        pre_list = filtered_df["model"].unique()
        for each_model in pre_list:
            model_list.append(each_model)
        return model_list

    @staticmethod
    def get_processor_br_list_by_model(model):
        filtered_df = Analysis.data[Analysis.data["model"] == model]
        processor_br_list = []
        pre_list = filtered_df["processor_brand"].unique()
        for each_processor_br in pre_list:
            processor_br_list.append(each_processor_br)
        return processor_br_list

    @staticmethod
    def get_processor_list_by_brand_model(brand, model):
        filtered_df = Analysis.data[(Analysis.data["brand"] == brand) & Analysis.data["model"] == model]
        processor_br_list = []
        pre_list = filtered_df["processor_brand"].unique()
        for each_processor_br in pre_list:
            processor_br_list.append(each_processor_br)
        return processor_br_list

    @staticmethod
    def search(brand, model, cpu_brand):
        filtered_df = Analysis.data[(Analysis.data["brand"] == brand) & (Analysis.data["model"] == model) &
                                    (Analysis.data["processor_brand"] == cpu_brand)]
        rescaled_df = filtered_df.reset_index(drop=True)
        return rescaled_df

    @staticmethod
    def histogram_price_graph():
        df = Analysis.data
        q1, q3 = df['Price'].quantile([0.25, 0.75])
        iqr = q3 - q1
        lower_bound = q1 - 1.5 * iqr
        upper_bound = q3 + 1.5 * iqr
        df = df[(df['Price'] >= lower_bound) & (df['Price'] <= upper_bound)]
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.hist(df["Price"], bins=5)
        ax.set_title("Histogram of Price")
        ax.set_xlabel("Price (Baht)")
        ax.set_ylabel("Frequency")
        ax.grid(axis="y")
        mean = df["Price"].mean()
        median = df["Price"].median()
        mode = df["Price"].mode()[0]
        std_dev = df["Price"].std()
        return fig, ax, mean, median, mode, std_dev

    @staticmethod
    def histogram_star_rating_graph():
        df = Analysis.data
        df = df[df['StarRate'] != 0]
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.hist(df["StarRate"], bins=5)
        ax.set_title("Histogram of Star Rating")
        ax.set_xlabel("Rating")
        ax.set_ylabel("Frequency")
        ax.grid(axis="y")
        mean = df["StarRate"].mean()
        median = df["StarRate"].median()
        mode = df["StarRate"].mode()[0]
        std_dev = df["StarRate"].std()
        return fig, ax, mean, median, mode, std_dev

    @staticmethod
    def scatter_price_vs_star_rate():
        df = Analysis.data
        q1, q3 = df['Price'].quantile([0.25, 0.75])
        iqr = q3 - q1
        lower_bound = q1 - 1.5 * iqr
        upper_bound = q3 + 1.5 * iqr
        df = df[(df['Price'] >= lower_bound) & (df['Price'] <= upper_bound)]
        df = df[df['StarRate'] != 0]
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.scatter(df["Price"], df["StarRate"])
        ax.set_title("Scatter Plot of Price vs Star Rating")
        ax.set_xlabel("Price")
        ax.set_ylabel("Star Rating")
        # Add trend line
        x = df["Price"]
        y = df["StarRate"]
        z = np.polyfit(x, y, 1)
        p = np.poly1d(z)
        ax.plot(x, p(x), "r--")
        corr = df[["Price", "StarRate"]].corr()
        corr_value = corr.iloc[0, 1]
        return fig, ax, corr_value

    @staticmethod
    def pie_chart_of_brand():
        df = Analysis.data
        brand_counts = df["brand"].value_counts(normalize=True)
        other_pct = brand_counts[brand_counts < 0.01].sum()
        brand_counts = brand_counts[brand_counts >= 0.01]
        brand_counts.loc["Other"] = other_pct
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        colors = ["#ff9999", "#66b3ff", "#99ff99", "#ffcc99", "#ffa07a"]
        ax.pie(brand_counts, labels=brand_counts.index, autopct='%1.1f%%', colors=colors)
        ax.set_title("Pie chart of laptop brand")
        return fig, ax

    @staticmethod
    def star_rating_each_brand():
        df = Analysis.data
        df = df[df['StarRate'] != 0]
        brand_ratings = df.groupby('brand')['StarRate'].mean().sort_values(ascending=False)
        fig = Figure(figsize=(8, 6), dpi=100)
        ax = fig.add_subplot(111)
        ax.barh(brand_ratings.index, brand_ratings.values)
        ax.set_title("Average Star Rating by Brand")
        ax.set_xlabel("Average Star Rating")
        ax.set_ylabel("Brand")
        ax.grid(axis="x")
        return fig, ax

    @staticmethod
    def box_plot_price():
        df = Analysis.data
        q1, q3 = df['Price'].quantile([0.25, 0.75])
        iqr = q3 - q1
        lower_bound = q1 - 1.5 * iqr
        upper_bound = q3 + 1.5 * iqr
        df = df[(df['Price'] >= lower_bound) & (df['Price'] <= upper_bound)]
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.boxplot(df["Price"])
        ax.set_title("Box Plot of Price")
        ax.set_xlabel("Price (Baht)")
        mean = df["Price"].mean()
        median = df["Price"].median()
        mode = df["Price"].mode()[0]
        std_dev = df["Price"].std()
        return fig, ax, mean, median, mode, std_dev

    @staticmethod
    def box_plot_star_rate():
        df = Analysis.data
        df = df[df['StarRate'] != 0]
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.boxplot(df["StarRate"])
        ax.set_title("Box Plot of Star Rating")
        ax.set_xlabel("Rating")
        mean = df["StarRate"].mean()
        median = df["StarRate"].median()
        mode = df["StarRate"].mode()[0]
        std_dev = df["StarRate"].std()
        return fig, ax, mean, median, mode, std_dev

    @staticmethod
    def bar_brand_by_user(data):
        df = data
        fig = Figure(figsize=(5, 4), dpi=100)
        ax = fig.add_subplot(111)
        ax.hist(df["Price"], bins=5)
        ax.set_title("Histogram of Price by your scope data")
        ax.set_xlabel("Price (Baht)")
        ax.set_ylabel("Frequency")
        ax.grid(axis="y")
        mean = df["Price"].mean()
        median = df["Price"].median()
        mode = df["Price"].mode()[0]
        std_dev = df["Price"].std()
        return fig, ax, mean, median, mode, std_dev
