import tkinter as tk
from tkinter import ttk
import pandas as pd
import pandastable as pt
from GuiTools import GuiTools
from Analysis import Analysis
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg


class GUI:
    def __init__(self):
        # Create a root name is Window and set attribute
        self.window = tk.Tk()
        self.window.geometry("1000x600")
        self.window.title("Laptops Analysis")
        self.window.resizable(False, False)
        # Important attribute
        self.__main_dataframe = None
        self.is_dataframe_exist = tk.BooleanVar(value=False)
        # create the control zone frame on the left
        self.control_frame = tk.Frame(self.window, width=400, height=600, bg="white")
        self.control_frame.pack(side=tk.LEFT, fill=tk.Y)
        self.control_frame.pack_propagate(False)
        # create the output zone frame on the right
        self.output_frame = tk.Frame(self.window, width=600, height=600, bg="light grey")
        self.output_frame.pack(side=tk.RIGHT)
        self.output_frame.pack_propagate(False)
        # controller zone
        self.search_menu_bar()  # Create a search bar menu
        self.label_of_founded_data()
        self.show_dataframe()
        # User option buttons area of Frame
        button_user_option_frame = tk.Frame(self.control_frame)
        button_user_option_frame.grid(row=3)
        button_user_option_frame.pack_propagate(False)
        # Histogram of price
        self.histogram_price_btn = tk.Button(button_user_option_frame, text="Histogram of price\n"
                                                                            "on Database", **GuiTools.button_style)
        self.histogram_price_btn.grid(row=0, column=0, padx=10, pady=5)
        self.histogram_price_btn.config(command=GUI.plot_price_hist_func)
        # Histogram of Star Rating
        self.histogram_star_rate_btn = tk.Button(button_user_option_frame, text="Histogram of Star rating\n"
                                                                                "on Database", **GuiTools.button_style)
        self.histogram_star_rate_btn.grid(row=1, column=0, padx=10, pady=5)
        self.histogram_star_rate_btn.config(command=GUI.plot_star_rating_hist_func)
        # Pie chart of brand in Database
        self.pie_brand_btn = tk.Button(button_user_option_frame, text="Pie chart of brand", **GuiTools.button_style)
        self.pie_brand_btn.grid(row=0, column=1, padx=10, pady=5)
        self.pie_brand_btn.config(command=GUI.show_pie_chart)
        # bar graph to Star Rating of each laptop's brand
        self.bar_graph_star_brand_btn = tk.Button(button_user_option_frame, text="Bar graph of star rate \n each brand",
                                                  **GuiTools.button_style)
        self.bar_graph_star_brand_btn.grid(row=1, column=1, padx=10, pady=5)
        self.bar_graph_star_brand_btn.config(command=GUI.show_bar_graph)
        # box plot of Price
        self.box_plot_price_button = tk.Button(button_user_option_frame, text="Box plot of Price",
                                               **GuiTools.button_style)
        self.box_plot_price_button.grid(row=2, column=0, padx=10, pady=5)
        self.box_plot_price_button.config(command=GUI.box_plot_price_func)
        # box plot of Star Rating
        self.box_plot_star_button = tk.Button(button_user_option_frame, text="Box plot of StarRating",
                                              **GuiTools.button_style)
        self.box_plot_star_button.grid(row=2, column=1, padx=10, pady=5)
        self.box_plot_star_button.config(command=GUI.box_plot_star_rate_func)
        # Scatter graph of Price and Star Rating
        self.scatter_plot_btn = tk.Button(button_user_option_frame, text="Scatter graph\nPrice and Star Rating",
                                          **GuiTools.button_style)
        self.scatter_plot_btn.grid(row=3, column=0, padx=10, pady=5)
        self.scatter_plot_btn.config(command=GUI.plot_scatter_price_star_rate_func)
        # Bar graph of brand by your data
        self.bar_brand_user_button = tk.Button(button_user_option_frame, text="Histogram of Price \n your scope data",
                                               **GuiTools.button_style)
        self.bar_brand_user_button.grid(row=3, column=1, padx=10, pady=5)
        self.is_dataframe_exist.trace("w", lambda *args: self.update_bar_brand_user_button_state())

    # this a method to create 3 combobox to scope(searching) dataframe
    def search_menu_bar(self):
        frame = tk.Frame(self.control_frame, bg="white")
        frame.grid(row=1)
        text_warning = tk.Label(frame, bg="white", text="Search your laptops", font=("Helvetica", 20))
        text_warning.pack(pady=10)
        search_bar_label = tk.Label(frame, bg="white")
        search_bar_label.pack(pady=10)
        # create Combobox of brand
        brand_list = Analysis.get_brand_list()
        brand_selection = tk.StringVar()
        brand_menu = GuiTools.create_menu_combobox(search_bar_label, brand_selection, brand_list)
        brand_menu.pack(side="left", padx=5)
        # create Combobox of model
        model_selection = tk.StringVar()
        model_menu = GuiTools.create_menu_combobox(search_bar_label, model_selection, [])
        model_menu.pack(side="left", padx=5)
        # create Combobox of processor
        processor_selection = tk.StringVar()
        processor_menu = GuiTools.create_menu_combobox(search_bar_label, processor_selection, [])
        processor_menu.pack(side="left", padx=5)

        def update_model_list(*args):
            model_list = Analysis.get_model_list_by_brand(brand_selection.get())
            model_menu.configure(values=model_list)
            if model_selection.get() not in model_list:
                model_selection.set('')
            update_processor_list()

        brand_selection.trace("w", update_model_list)

        def update_processor_list(*args):
            if not args:
                args = ()
            processor_list = Analysis.get_processor_br_list_by_model(model_selection.get())
            processor_menu.configure(values=processor_list)
            if processor_selection.get() not in processor_list:
                processor_selection.set('')

        model_selection.trace("w", update_processor_list)
        brand_selection.trace("w", update_processor_list)

        def update_model_and_processor(*args):
            update_model_list()
            update_processor_list()

        processor_selection.trace("w", update_model_and_processor)

        def set_button_function():  # set main data frame
            dataframe = Analysis.search(brand_selection.get(), model_selection.get(), processor_selection.get())
            self.set_main_dataframe(dataframe)
            print(dataframe)

        def reset_button_function():  # function to reset 3 Combobox and reset main_dataframe to none
            brand_selection.set('')
            model_selection.set('')
            processor_selection.set('')
            self.reset_main_dataframe()

        # create click button area
        click_label = tk.Label(frame, bg="white")
        click_label.pack(pady=10)
        show_button = tk.Button(click_label, text="Show", command=set_button_function, **GuiTools.button_style)
        show_button.pack(side="left", padx=5)
        reset_button = tk.Button(click_label, text="Reset", command=reset_button_function, **GuiTools.button_style)
        reset_button.pack(side="left", padx=5)

    # this method create area of monitor to show a searching or scoping result
    def label_of_founded_data(self):
        text_frame = tk.Frame(self.control_frame, bg="light grey", width=400, height=100)
        text_frame.grid(row=2, pady=25)
        text_frame.pack_propagate(False)
        label1 = tk.Label(text_frame, bg="light grey", font=("Arial", 20), text="No Data")
        label1.pack(pady=10)
        label2 = tk.Label(text_frame, bg="light grey", font=("Arial", 15))
        label2.pack(pady=10)

        def update_labels():  # update a monitor(label)
            if self.is_dataframe_exist.get() is True:
                label1.config(text="Founded")
                text = f"{len(self.__main_dataframe)} result"
                label2.config(text=text)
            else:
                label1.config(text="No Data")
                label2.config(text="")
        update_labels()
        self.is_dataframe_exist.trace_add("write", lambda *args: update_labels())  # make it update real time

    def show_dataframe(self):  # this method is created a table to show data after scoping or searching
        table_frame = tk.Frame(self.output_frame, bg="white")
        table_frame.pack(fill=tk.BOTH, expand=True, pady=20, padx=20)
        # Create the initial table
        model = pt.TableModel(dataframe=pd.DataFrame())
        table = pt.Table(table_frame, model=model)
        table.show()
        table.grid()

        def update_table_frame():
            if self.is_dataframe_exist is False:
                new_model = pt.TableModel(dataframe=pd.DataFrame())
            else:
                new_model = pt.TableModel(dataframe=self.__main_dataframe)
            table.updateModel(new_model)  # update the existing table with the new model
            table.redraw()
        update_table_frame()
        # update realtime when values change by trac_add
        self.is_dataframe_exist.trace_add("write", lambda *args: update_table_frame())

    def update_bar_brand_user_button_state(self):
        if self.is_dataframe_exist:
            self.bar_brand_user_button.config(command=self.bar_brand_user_func)
        else:
            self.bar_brand_user_button.config(command=NotImplemented)

    @staticmethod
    def plot_price_hist_func():
        window = GuiTools.create_toplevel("Histogram of Price")
        fig, ax, mean, median, mode, std_dev = Analysis.histogram_price_graph()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Mean: {mean:.2f}, Median: {median}, Mode: {mode}, Std Dev: {std_dev:.2f}",
                          font=("Arial", 13))
        label.pack(side='bottom')

    @staticmethod
    def plot_star_rating_hist_func():
        window = GuiTools.create_toplevel("Histogram of Star Rating")
        fig, ax, mean, median, mode, std_dev = Analysis.histogram_star_rating_graph()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Mean: {mean:.3f}, Median: {median:.3f}, Mode: {mode}, Std Dev: {std_dev:.3f}",
                          font=("Arial", 13))
        label.pack(side='bottom')

    @staticmethod
    def plot_scatter_price_star_rate_func():
        window = GuiTools.create_toplevel("Scatter graph of Price and Star Rating")
        fig, ax, corr = Analysis.scatter_price_vs_star_rate()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Correlation is {corr:.3f}", font=("Arial", 13))
        label.pack(side='bottom')

    @staticmethod
    def show_pie_chart():
        window = GuiTools.create_toplevel("Pie chart of brand")
        fig, ax = Analysis.pie_chart_of_brand()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)

    @staticmethod
    def show_bar_graph():
        window = GuiTools.create_toplevel("Bar graph of brand")
        fig, ax = Analysis.star_rating_each_brand()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)

    @staticmethod
    def box_plot_price_func():
        window = GuiTools.create_toplevel("Box plot of Price")
        fig, ax, mean, median, mode, std_dev = Analysis.box_plot_price()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Mean: {mean:.2f}, Median: {median}, Mode: {mode}, Std Dev: {std_dev:.2f}",
                          font=("Arial", 13))
        label.pack(side='bottom')

    @staticmethod
    def box_plot_star_rate_func():
        window = GuiTools.create_toplevel("Box plot of Star Rating")
        fig, ax, mean, median, mode, std_dev = Analysis.box_plot_star_rate()
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Mean: {mean:.3f}, Median: {median:.3f}, Mode: {mode}, Std Dev: {std_dev:.3f}",
                          font=("Arial", 13))
        label.pack(side='bottom')

    def bar_brand_user_func(self):
        window = GuiTools.create_toplevel("Histogram of Price by your scope data")
        fig, ax, mean, median, mode, std_dev = Analysis.bar_brand_by_user(self.__main_dataframe)
        canvas = FigureCanvasTkAgg(fig, master=window)
        canvas.draw()
        canvas.get_tk_widget().pack(side='top', fill='both', expand=1)
        label = ttk.Label(window, text=f"Mean: {mean:.3f}, Median: {median:.3f}, Mode: {mode}, Std Dev: {std_dev:.3f}",
                          font=("Arial", 13))
        label.pack(side='bottom')

    def reset_main_dataframe(self):  # reset method to set main_dataframe to None
        self.__main_dataframe = None
        self.is_dataframe_exist.set(False)

    def get_main_dataframe(self):  # getter method of main_dataframe
        return self.__main_dataframe

    def set_main_dataframe(self, new): # setter method of main_dataframe
        self.__main_dataframe = new
        self.is_dataframe_exist.set(True)

    def run(self):
        self.window.mainloop()
